import { world, ItemStack, GameMode } from '@minecraft/server';

// This array defines the vanilla items that will lose durability when mining an addon block.
const vanillaToolIds = [
    'minecraft:wooden_sword',
    'minecraft:wooden_shovel',
    'minecraft:wooden_pickaxe',
    'minecraft:wooden_axe',
    'minecraft:wooden_hoe',
    'minecraft:stone_sword',
    'minecraft:stone_shovel',
    'minecraft:stone_pickaxe',
    'minecraft:stone_axe',
    'minecraft:stone_hoe',
    'minecraft:iron_sword',
    'minecraft:iron_shovel',
    'minecraft:iron_pickaxe',
    'minecraft:iron_axe',
    'minecraft:iron_hoe',
    'minecraft:golden_sword',
    'minecraft:golden_shovel',
    'minecraft:golden_pickaxe',
    'minecraft:golden_axe',
    'minecraft:golden_hoe',
    'minecraft:diamond_sword',
    'minecraft:diamond_shovel',
    'minecraft:diamond_pickaxe',
    'minecraft:diamond_axe',
    'minecraft:diamond_hoe',
    'minecraft:netherite_sword',
    'minecraft:netherite_shovel',
    'minecraft:netherite_pickaxe',
    'minecraft:netherite_axe',
    'minecraft:netherite_hoe',
    'minecraft:shears'
];

world.afterEvents.playerBreakBlock.subscribe(evd => {
    const { player, brokenBlockPermutation: block, itemStackBeforeBreak: itemUsed } = evd;

    // This returns if itemUsed is undefined or if the player is in Creative.
    if (!itemUsed || player.matches({ gameMode: GameMode.creative })) return;

    // Checks if the blocks being mined were added by the addon.
    if (block.matches('azonix:frobo_coral_bottom') || block.matches('azonix:frobo_coral_top') || block.matches('azonix:frobo_fern_bottom') || block.matches('azonix:frobo_fern_top') || block.matches('azonix:bloom_erylium') || block.matches('azonix:bloom_vine') || block.matches('azonix:chorus_lamp') || block.matches('azonix:chorus_vine') || block.matches('azonix:tall_rafflesia_bottom') || block.matches('azonix:tall_rafflesia_top') || block.matches('azonix:engrove_clover.fruit') || block.matches('azonix:engrove_clover') || block.matches('azonix:engrove_erylium') || block.matches('azonix:engrove_growths') || block.matches('azonix:engrove_moss') || block.matches('azonix:engrove_quills') || block.matches('azonix:engrove_roots') || block.matches('azonix:engrove_vine_body') || block.matches('azonix:blomp_goo.baby_blue') || block.matches('azonix:blomp_goo.baby_pink') || block.matches('azonix:blomp_goo.baby_purple') || block.matches('azonix:blomp_goo.blue') || block.matches('azonix:blomp_goo.pink') || block.matches('azonix:blomp_goo.purple') || block.matches('azonix:chorus_bush_bottom') || block.matches('azonix:chorus_bush_top') || block.matches('azonix:bounce_berry') || block.matches('azonix:bounce_flower') || block.matches('azonix:bounce_vine_body') || block.matches('azonix:bounce_vine_head') || block.matches('azonix:cobbled_voidstone') || block.matches('azonix:compressed_ice') || block.matches('azonix:end_grass') || block.matches('azonix:enderlock.actived') || block.matches('azonix:enderlock') || block.matches('azonix:polished_voidstone_bricks') || block.matches('azonix:spectral_goo_block') || block.matches('azonix:voidstone') || block.matches('azonix:voidstone_bricks') || block.matches('azonix:zircollium_block') || block.matches('azonix:zircon_block') || block.matches('azonix:zircon_crystal') || block.matches('azonix:zircon_crystal_to_air_temp_block') || block.matches('azonix:zircon_to_zircollium_temp_block') || block.matches('azonix:kyrie_ore') || block.matches('azonix:raw_tanzanite_block') || block.matches('azonix:tanzanite_block') || block.matches('azonix:tanzanite_ore') || block.matches('azonix:corpse_flower_bottom') || block.matches('azonix:corpse_flower_top') || block.matches('azonix:jam_coral_bottom') || block.matches('azonix:jam_coral_top') || block.matches('azonix:sodium_plant_bottom') || block.matches('azonix:sodium_plant_middle') || block.matches('azonix:sodium_plant_top') || block.matches('azonix:jakra_vine_body') || block.matches('azonix:jakra_vine_head') || block.matches('azonix:jam_fern') || block.matches('azonix:jam_grass') || block.matches('azonix:jellix') || block.matches('azonix:jelly_erylium') || block.matches('azonix:bloom_door') || block.matches('azonix:bloom_fence') || block.matches('azonix:bloom_fence_gate') || block.matches('azonix:bloom_leaves') || block.matches('azonix:bloom_log') || block.matches('azonix:bloom_planks') || block.matches('azonix:bloom_sapling') || block.matches('azonix:bloom_slab') || block.matches('azonix:bloom_stairs') || block.matches('azonix:engrove_door') || block.matches('azonix:engrove_fence') || block.matches('azonix:engrove_fence_gate') || block.matches('azonix:engrove_leaves') || block.matches('azonix:engrove_log') || block.matches('azonix:engrove_planks') || block.matches('azonix:engrove_slab') || block.matches('azonix:engrove_stairs') || block.matches('azonix:jellight') || block.matches('azonix:puffle_door') || block.matches('azonix:puffle_fence') || block.matches('azonix:puffle_fence_gate') || block.matches('azonix:puffle_log') || block.matches('azonix:puffle_planks') || block.matches('azonix:puffle_sapling') || block.matches('azonix:puffle_slab') || block.matches('azonix:puffle_stairs')) {
        if (vanillaToolIds.includes(itemUsed.typeId)) {

            // This retrieves the player's equippable component.
            const playerEquippableComp = player.getComponent("equippable");

            // This returns if playerEquippableComp is undefined.
            if (!playerEquippableComp) return;

            // This retrieves the enchantable component of the item.
            const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
            const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;

            // Calculates the chance of an item breaking based on its unbreaking level. This is the vanilla unbreaking formula.
            const breakChance = 100 / (unbreakingLevel + 1);
            // Generates a random chance value between 0 and 100.
            const randomizeChance = Math.random() * 100;

            // This returns if breakChance is less than randomizeChance.
            if (breakChance < randomizeChance) return;

            // This retrieves the durability component of the item.
            const itemUsedDurabilityComp = itemUsed.getComponent("durability");

            // This returns if itemUsedDurabilityComp is undefined.
            if (!itemUsedDurabilityComp) return;

            // This will set the new durability value.
            itemUsedDurabilityComp.damage += 1;

            // Declares and checks if the item is out of durability
            const maxDurability = itemUsedDurabilityComp.maxDurability
            const currentDamage = itemUsedDurabilityComp.damage
            if (currentDamage >= maxDurability) {

                // If the item is out of durability, plays the item breaking sound and removes the item
                player.playSound('random.break', { pitch: 1, location: player.location, volume: 1 })
                playerEquippableComp.setEquipment("Mainhand", new ItemStack('minecraft:air', 1));
            }
            else if (currentDamage < maxDurability) {
    
                // This sets the item in the player's selected slot.
                playerEquippableComp.setEquipment("Mainhand", itemUsed);
            }
        }
    }
})